module ResourcePlanner
  module Models

    # Creates configuration values for ResourcePlanner and for the given module.
    #
    #   ResourcePlanner::Models.config(ResourcePlanner::Resource, :colour, 10)
    #
    # The line above creates:
    #
    #   1) An accessor called ResourcePlanner.colour, which value is used by default;
    #
    #   2) Some class methods for your model Model.colour and Model.colour=
    #      which have higher priority than ResourcePlanner.colour;
    #
    #   3) And an instance method colour.
    #
    # To add the class methods you need to have a module ClassMethods defined
    # inside the given class.
    #
    def self.config(mod, *accessors) #:nodoc:
      class << mod; attr_accessor :available_configs; end
      mod.available_configs = accessors

      accessors.each do |accessor|
        mod.class_eval <<-METHOD, __FILE__, __LINE__ + 1
          def #{accessor}
            if defined?(@#{accessor})
              @#{accessor}
            elsif superclass.respond_to?(:#{accessor})
              superclass.#{accessor}
            else
              Devise.#{accessor}
            end
          end

          def #{accessor}=(value)
            @#{accessor} = value
          end
        METHOD
      end
    end

    # Include the chosen ResourcePlanner modules in your model:
    #
    #   resource_planner :resource, :reservable, :exclusive
    #
    # You can also give any of the devise configuration values in form of a hash,
    # with specific values for this model. Please check your ResourcePlanner initializer
    # for a complete description on those values.
    #
    def resource_planner(*modules)
      options = modules.extract_options!.dup

      selected_modules = modules.map(&:to_sym).uniq.sort_by do |s|
        ResourcePlanner::ALL.index(s) || -1  # follow ResourcePlanner::ALL order
      end

      resource_planner_modules_hook! do
        include ResourcePlanner::Models::Resourceable
        selected_modules.each do |m|
          if m == :encryptable && !(defined?(ResourcePlanner::Models::Encryptable))
            warn "[ResourcePlanner] You're trying to include :encryptable in your model but it is not bundled with the ResourcePlanner gem anymore. Please add `resource_planner-encryptable` to your Gemfile to proceed.\n"
          end

          mod = ResourcePlanner::Models.const_get(m.to_s.classify)

          if mod.const_defined?("ClassMethods")
            class_mod = mod.const_get("ClassMethods")
            extend class_mod

            if class_mod.respond_to?(:available_configs)
              available_configs = class_mod.available_configs
              available_configs.each do |config|
                next unless options.key?(config)
                send(:"#{config}=", options.delete(config))
              end
            end
          end

          include mod
        end

        self.devise_modules |= selected_modules
        options.each { |key, value| send(:"#{key}=", value) }
      end
    end

    # The hook which is called inside ResourcePlanner.
    # So your ORM can include devise compatibility stuff.
    def resource_planner_modules_hook!
      yield
    end

  end
end

require 'resource_planner/models/resourceable'
